
       //=======================================================//    _\|/_
      //  __  _____           ___                    ___       //      /|\
     //  /      |      ^     |   \  |         ^     |   \     //          _\|/_
    //   \__    |     / \    |___/  |        / \    |___/    //            /|\
   //       \   |    /___\   |  \   |       /___\   |   \   // _\|/_
  //     ___/   |   /     \  |   \  |____  /     \  |___/  //   /|\     
 //                                                       //            _\|/_
//=======================================================//              /|\

/*
 *  star.h: base class for star systems, on the same level as dyn
 *.............................................................................
 *    version 1:  Jan 1993   Piet Hut
 *    version 2:
 *.............................................................................
 *     This file includes:
 *  1) definition of class star
 *.............................................................................
 */

#ifndef  STARLAB_STAR_H
#  define  STARLAB_STAR_H

#include  "starlab_vector.h"
#include  "story.h"
#include  "starbase.h"
#include  "node.h"       // needed in functions such as get_effective_radius().
                         // It cannot be included in  starbase.h  since 
                         // node.h contains  starbase.h  and this would
                         // lead to a vicious circle

//#include "seba.h"
/*-----------------------------------------------------------------------------
 *  star  --  the simplest class for stellar evolution
 *-----------------------------------------------------------------------------
 */
class  star : public starbase
    {
    protected:

        real  current_age;
	real  main_sequence_lifetime;

	real  m_conv_star_to_dyn;   // mass conversion factor
	real  r_conv_star_to_dyn;   // length conversion factor
	real  t_conv_star_to_dyn;   // time conversion factor

//	static seba_counters sbc;

    public:

	star(real age = 0, real lifetime = 1,
	      real mf = 1, real rf = 1, real tf = 1)
	    {
	    current_age = age;
	    main_sequence_lifetime = lifetime;
	    m_conv_star_to_dyn = mf;
	    r_conv_star_to_dyn = rf;
	    t_conv_star_to_dyn = tf;
	    }

//      to convert an internal star mass variable m_star into the units used
//	by the dyn part, you can use:
//
//          a_dyn_particle->set_mass( m_star * get_m_conv_star_to_dyn() );

//	seba_counters get_seba_counters();
//	void set_seba_counters(seba_counters sb);

    	real  get_m_conv_star_to_dyn()      {return m_conv_star_to_dyn;}
    	real  get_r_conv_star_to_dyn()      {return r_conv_star_to_dyn;}
    	real  get_t_conv_star_to_dyn()      {return t_conv_star_to_dyn;}

    	real  get_current_age()               {return current_age;}

	real  get_ms_lifetime()               {return main_sequence_lifetime;}

    	void  set_m_conv_star_to_dyn(const real mf)
	    {m_conv_star_to_dyn = mf;}
    	void  set_r_conv_star_to_dyn(const real rf)
	    {r_conv_star_to_dyn = rf;}
    	void  set_t_conv_star_to_dyn(const real tf)
	    {t_conv_star_to_dyn = tf;}

	void  set_current_age(const real m)   {current_age = m;}

	void  set_ms_lifetime(const real r)   {main_sequence_lifetime=r;}

	void  inc_current_age(const real dt)  {current_age += dt;}

	virtual ostream & print_star_story(ostream&,
					   int short_output = 0);
	virtual istream & scan_star_story(istream&);
    };

inline  starbase * new_star()    {return dynamic_cast(starbase *, new star);}

// Shorthand for conversion from node pointer to star pointer:

#define N_S_PTR (dynamic_cast(star *, n->get_starbase()))

// note: automatic conversion from star to dyn scaling

inline  real get_ms_lifetime(node * n)
    {return N_S_PTR->get_ms_lifetime() * N_S_PTR->get_t_conv_star_to_dyn();}
    
inline  real get_m_conv_star_to_dyn(node * n)
    {return N_S_PTR->get_m_conv_star_to_dyn();}
inline  real get_r_conv_star_to_dyn(node * n)
    {return N_S_PTR->get_r_conv_star_to_dyn();}
inline  real get_t_conv_star_to_dyn(node * n)
    {return N_S_PTR->get_t_conv_star_to_dyn();}

inline  void set_m_conv_star_to_dyn(node * n, const real r)
    {N_S_PTR->set_m_conv_star_to_dyn(r);}
inline  void set_r_conv_star_to_dyn(node * n, const real r)
    {N_S_PTR->set_r_conv_star_to_dyn(r);}
inline  void set_t_conv_star_to_dyn(node * n, const real r)
    {N_S_PTR->set_t_conv_star_to_dyn(r);}
    
// note: automatic conversion from dyn to star scaling

inline  void set_ms_lifetime(node * n, const real t)
    {N_S_PTR->set_ms_lifetime( t / N_S_PTR->get_t_conv_star_to_dyn());}
    
inline  real get_current_age(node * n)
    {return N_S_PTR->get_current_age() * N_S_PTR->get_t_conv_star_to_dyn();}
    
inline  void set_current_age(node * n, const real t)
    {N_S_PTR->set_current_age( t / N_S_PTR->get_t_conv_star_to_dyn());}
    
inline  void inc_current_age(node * n, const real dt)
    {N_S_PTR->inc_current_age( dt / N_S_PTR->get_t_conv_star_to_dyn());}

void  addstar(node *, real, real, bool verbose = false);

#endif
 
