/*
 * wolf_rayet.h: derived class for evolution of Wolf-Rayet stars.
 *
 *.....................................................................
 *    version 1:  Jan 1994   Simon F. Portegies Zwart
 *    version 2:
 *...................................................................
 *     This file includes:
 *  1) definition of class wolf_rayet
 *
 *....................................................................
 */

#ifndef   _WOLF_RAYET 
#   define _WOLF_RAYET 

#include "single_star.h"
#include "helium_star.h"
#include "neutron_star.h"
#include "black_hole.h"

class main_sequence;

/*-----------------------------------------------------------------------------
 *  wolf_rayet  --  a derived class for element evolution.
 *-----------------------------------------------------------------------------
 */
class wolf_rayet : public single_star {
      private:

          real wolf_rayet_core_mass();

      public :

         wolf_rayet(main_sequence &);
         wolf_rayet(node* n) : single_star(n) {}
         ~wolf_rayet() {}

	 
         stellar_type get_element_type() {return Wolf_Rayet;}
         bool giant_star()             {return true;}
         bool hydrogen_envelope_star() {return false;}

         void instantaneous_element();
         void evolve_element(const real);
         void create_remnant();
	 
//            Mass transfer utilities.
	 star* subtrac_mass_from_donor(const real, real&);
         real accretion_limit(const real, const real);

         void adjust_accretor_age(const real, const bool);
         void adjust_next_update_age();
         star* reduce_mass(const real);

//              Stability rourines.
         real zeta_thermal();
         real zeta_adiabatic();
	 real gyration_radius_sq();

	 friend helium_star::helium_star(wolf_rayet &);
	 friend neutron_star::neutron_star(wolf_rayet &);
	 friend black_hole::black_hole(wolf_rayet &);
    };
#endif 		// _WOLF_RAYET 


//         real zeta_adiabatic();
//         void adjust_initial_star();
//         real stellar_radius(const real m, const real t) 
//              {return 5;}
