
//  compute_com.C: determine the center of mass of an Nbody system.
//		   When used as a pipelined tool, all information is
//		   returned in the dyn story of the top-level node.

#include "dyn.h"
#include "program.h"

#ifndef TOOLBOX

void compute_com(dyn *b, vector& pos, vector& vel)
{
    real total_mass = 0;
    pos = 0;
    vel = 0;
    
//  for_all_leaves(dyn, b, d)
    for_all_daughters(dyn, b, d) {
	total_mass += d->get_mass();
	pos += d->get_mass() * d->get_pos();
	vel += d->get_mass() * d->get_vel();
    }	

    pos /= total_mass;
    vel /= total_mass;
}

#else

/*-----------------------------------------------------------------------------
 *  main  --  driver to use compute_com() as a tool
 *-----------------------------------------------------------------------------
 */

main(int argc, char ** argv)
{
    int  c;
    char  *comment;
    extern char *poptarg;
    dyn * b;
    bool  c_flag = FALSE;       /* if TRUE: a comment given on command line  */
    int  pgetopt(int, char **, char *);

    while ((c = pgetopt(argc, argv, "c:")) != -1)
	switch(c) {

	    case 'c': c_flag = TRUE;
		      comment = poptarg;
		      break;
            case '?': err_exit("usage: compute_com [-c]");
        }            

    if ((b = get_dyn(cin)) == NULL)
       err_exit("compute_com: No N-body system on standard input");

    while (b) {

        if (c_flag == TRUE)
            b->log_comment(comment);
        b->log_history(argc, argv);

	vector pos, vel;
        compute_com(b, pos, vel);

	putvq(b->get_dyn_story(), "com_pos", pos);
	putvq(b->get_dyn_story(), "com_vel", vel);

	// Write system to stdout and get next system (if any).

        put_dyn(cout, *b);
	rmtree(b);
	b = get_dyn(cin);
    }
}

#endif

/*===========================================================================*/

/* endof: compute_com.C */
